import asyncio

from yatracker import YaTracker
from yatracker.types import FullIssue

# CAUTION! Don't store credentials in your code!
ORG_ID = ...
TOKEN = ...


# Create your own custom Issue type:
class HelpIssue(FullIssue, kw_only=True):
    """Your own FullIssue type.

    For example, you have some fields passed by external system.
    One of them called 'userUsername', second - 'userId'.
    """

    user_username: str | None
    user_id: int


async def main() -> None:
    """Run example."""
    # define tracker (once)
    tracker = YaTracker(ORG_ID, TOKEN)

    # create an issue
    issue = await tracker.create_issue(
        summary="New Issue",
        queue="KEY",
        user_id=1234567890,
        _type=HelpIssue,
    )
    print(issue.user_id)

    # don't forget to close tracker on app shutdown (once)
    await tracker.close()


if __name__ == "__main__":
    asyncio.run(main())



from __future__ import annotations

__all__ = ["QueueVersion"]


from datetime import date

from .base import Base, field
from .queue import Queue


class QueueVersion(Base, kw_only=True):
    url: str = field(name="self")
    id: int
    version: int
    queue: Queue
    name: str
    description: str | None = None
    start_date: date | None = None
    due_date: date | None = None
    released: bool
    archived: bool



from __future__ import annotations

__all__ = ["Resolution"]


from .base import Base, field


class Resolution(Base, kw_only=True):
    url: str = field(name="self")
    id: str
    key: str
    display: str



from __future__ import annotations

__all__ = ["IssueTypeConfig"]


from .base import Base
from .issue_type import IssueType
from .resolution import Resolution
from .workflow import Workflow


class IssueTypeConfig(Base, kw_only=True):
    issue_type: IssueType
    workflow: Workflow
    resolutions: list[Resolution]



from __future__ import annotations

__all__ = ["QueueFieldOptionsProvider"]


from .base import Base


class QueueFieldOptionsProvider(Base, kw_only=True):
    type: str
    values: list
    defaults: list



__all__ = [
    "Attachment",
    "Base",
    "Comment",
    "Duration",
    "field",
    "FullIssue",
    "FullQueue",
    "Issue",
    "IssueType",
    "IssueTypeConfig",
    "Priority",
    "Queue",
    "QueueField",
    "QueueVersion",
    "Sprint",
    "Status",
    "Transition",
    "Transitions",
    "User",
    "Worklog",
]

from .attachment import Attachment
from .base import Base, field
from .comment import Comment
from .duration import Duration
from .full_issue import FullIssue
from .full_queue import FullQueue
from .issue import Issue
from .issue_type import IssueType
from .issue_type_config import IssueTypeConfig
from .priority import Priority
from .queue import Queue
from .queue_field import QueueField
from .queue_version import QueueVersion
from .sprint import Sprint
from .status import Status
from .transition import Transition
from .transitions import Transitions
from .user import User
from .worklog import Worklog



from __future__ import annotations

__all__ = ["QueueField"]


from .base import Base, field
from .queue_field_options_provider import QueueFieldOptionsProvider
from .queue_field_query_provider import QueueFieldQueryProvider
from .queue_field_schema import QueueFieldSchema


class QueueField(Base, kw_only=True):
    url: str = field(name="self")
    id: str
    name: str
    version: int

    field_schema: QueueFieldSchema
    readonly: bool
    options: bool
    suggest: bool
    options_provider: QueueFieldOptionsProvider | None = None
    query_provider: QueueFieldQueryProvider | None = None
    order: int



from __future__ import annotations

__all__ = ["FullQueue"]

from .base import Base, field
from .issue_type import IssueType
from .issue_type_config import IssueTypeConfig
from .priority import Priority
from .queue_version import QueueVersion
from .user import User
from .workflow import Workflow


class FullQueue(Base, kw_only=True):
    url: str = field(name="self")
    id: int
    key: str
    version: int

    name: str
    description: str | None = None
    lead: User
    assign_auto: bool
    default_type: IssueType
    default_priority: Priority
    team_users: list[User]
    issue_types: list[IssueType]
    versions: list[QueueVersion]
    workflows: list[Workflow]
    deny_voting: bool
    issue_types_config: list[IssueTypeConfig]



from __future__ import annotations

from typing import TypeVar, overload

from yatracker.tracker.base import BaseTracker
from yatracker.types import (
    FullQueue,
    IssueTypeConfig,
    QueueField,
    QueueVersion,
)

QueueT_co = TypeVar("QueueT_co", bound=FullQueue, covariant=True)
QueueFieldT_co = TypeVar("QueueFieldT_co", bound=QueueField, covariant=True)
QueueVersionT_co = TypeVar("QueueVersionT_co", bound=QueueVersion, covariant=True)


class Queues(BaseTracker):
    @overload
    async def get_queue(
        self,
        queue_id: str | int,
    ) -> FullQueue:
        ...

    @overload
    async def get_queue(
        self,
        queue_id: str | int,
        _type: type[QueueT_co] = ...,
    ) -> QueueT_co:
        ...

    async def get_queue(
        self,
        queue_id: str | int,
        _type: type[QueueT_co | FullQueue] = FullQueue,
    ) -> QueueT_co | FullQueue:
        """Get queue parameters.

        Use this request to get information about a queue.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/queues/get-queue

        :param queue_id: ID or key of the current queue.
        :param _type: you can use your own extended FullQueue type
        :return:
        """
        data = await self._client.request(
            method="GET",
            uri=f"/queues/{queue_id}",
        )
        return self._decode(_type, data)

    # ruff: noqa: PLR0913
    @overload
    async def create_queue(
        self,
        key: str,
        name: str,
        lead: str,
        default_type: str,
        default_priority: str,
        issue_types_config: list[IssueTypeConfig],
    ) -> FullQueue:
        ...

    # ruff: noqa: PLR0913
    @overload
    async def create_queue(
        self,
        key: str,
        name: str,
        lead: str,
        default_type: str,
        default_priority: str,
        issue_types_config: list[IssueTypeConfig],
        _type: type[QueueT_co] = ...,
    ) -> QueueT_co:
        ...

    # ruff: noqa: PLR0913
    async def create_queue(
        self,
        key: str,
        name: str,
        lead: str,
        default_type: str,
        default_priority: str,
        issue_types_config: list[IssueTypeConfig],
        _type: type[QueueT_co | FullQueue] = FullQueue,
    ) -> QueueT_co | FullQueue:
        """Create a queue.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/queues/create-queue
        """
        payload = self._prepare_payload(locals(), type_=_type)
        data = await self._client.request(
            method="POST",
            uri="/queues",
            payload=payload,
        )
        return self._decode(_type, data)

    @overload
    async def get_queues(
        self,
        expand: str | None = None,
        per_page: int | None = None,
    ) -> list[FullQueue]:
        ...

    @overload
    async def get_queues(
        self,
        expand: str | None = None,
        per_page: int | None = None,
        _type: type[QueueT_co] = ...,
    ) -> list[QueueT_co]:
        ...

    async def get_queues(
        self,
        expand: str | None = None,
        per_page: int | None = None,
        _type: type[FullQueue | QueueT_co] = FullQueue,
    ) -> list[FullQueue] | list[QueueT_co]:
        """Get queues.

        Use this request to get a list of available queues.
        If there are more than 50 queues in the response, use pagination.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/queues/get-queues
        """
        params = {}
        if expand is not None:
            params["expand"] = expand
        if per_page is not None:
            params["perPage"] = str(per_page)

        payload = self._prepare_payload(
            locals(),
            exclude=["expand", "perPage"],
            type_=_type,
        )
        data = await self._client.request(
            method="GET",
            uri="/queues",
            params=params,
            payload=payload,
        )
        return self._decode(list[_type], data)  # type: ignore[valid-type]

    async def delete_queue(
        self,
        queue_id: str | int,
    ) -> bool:
        """Delete queue.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/queues/delete-queue
        """
        await self._client.request(
            method="DELETE",
            uri=f"/queues/{queue_id}",
        )
        return True

    @overload
    async def restore_queue(
        self,
        queue_id: str | int,
    ) -> FullQueue:
        ...

    @overload
    async def restore_queue(
        self,
        queue_id: str | int,
        _type: type[QueueT_co] = ...,
    ) -> QueueT_co:
        ...

    async def restore_queue(
        self,
        queue_id: str | int,
        _type: type[QueueT_co | FullQueue] = FullQueue,
    ) -> QueueT_co | FullQueue:
        """Restore queue.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/queues/restore-queue
        """
        data = await self._client.request(
            method="GET",
            uri=f"/queues/{queue_id}/_restore",
        )
        return self._decode(_type, data)

    async def delete_tag_from_queue(
        self,
        queue_id: str | int,
        tag_name: str,
    ) -> bool:
        """Remove a tag from a queue.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/queues/delete-tag
        """
        await self._client.request(
            method="DELETE",
            uri=f"/queues/{queue_id}/tags/_remove",
            payload={"tag": tag_name},
        )
        return True

    @overload
    async def get_queue_fields(
        self,
        queue_id: str | int,
    ) -> list[QueueField]:
        ...

    @overload
    async def get_queue_fields(
        self,
        queue_id: str | int,
        _type: type[QueueFieldT_co] = ...,
    ) -> list[QueueFieldT_co]:
        ...

    async def get_queue_fields(
        self,
        queue_id: str | int,
        _type: type[QueueField | QueueFieldT_co] = QueueField,
    ) -> list[QueueField] | list[QueueFieldT_co]:
        """Get required fields for the queue.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/queues/get-fields
        """
        data = await self._client.request(
            method="GET",
            uri=f"/queues/{queue_id}/fields",
        )
        return self._decode(list[_type], data)  # type: ignore[valid-type]

    @overload
    async def get_queue_versions(
        self,
        queue_id: str | int,
    ) -> list[QueueVersion]:
        ...

    @overload
    async def get_queue_versions(
        self,
        queue_id: str | int,
        _type: type[QueueVersionT_co] = ...,
    ) -> list[QueueVersionT_co]:
        ...

    async def get_queue_versions(
        self,
        queue_id: str | int,
        _type: type[QueueVersion | QueueVersionT_co] = QueueVersion,
    ) -> list[QueueVersion] | list[QueueVersionT_co]:
        """Get queue versions.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/queues/get-versions
        """
        data = await self._client.request(
            method="GET",
            uri=f"/queues/{queue_id}/versions",
        )
        return self._decode(list[_type], data)  # type: ignore[valid-type]



from __future__ import annotations

__all__ = ["QueueFieldSchema"]


from .base import Base


class QueueFieldSchema(Base, kw_only=True):
    type: str
    required: bool | None = None
    items: str | None = None



from __future__ import annotations

from yatracker.tracker.base import BaseTracker
from yatracker.types import (
    Comment,
)


class Comments(BaseTracker):
    async def get_comments(self, issue_id: str) -> list[Comment]:
        """Get the comments for an issue.

        Use this request to get a list of comments in the issue.
        :param issue_id:
        :return:
        """
        data = await self._client.request(
            method="GET",
            uri=f"/issues/{issue_id}/comments",
        )
        return self._decode(list[Comment], data)

    async def post_comment(self, issue_id: str, text: str, **kwargs) -> Comment:
        """Comment the issue."""
        payload = self._prepare_payload(locals(), exclude=["issue_id"])
        data = await self._client.request(
            method="POST",
            uri=f"/issues/{issue_id}/comments/",
            payload=payload,
        )
        return self._decode(Comment, data)

    async def edit_comment(
        self,
        issue_id: str,
        comment_id: str | int,
        text: str,
        attachment_ids: list[str] | None = None,
    ) -> Comment:
        """Update issue comment.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/issues/edit-comment
        """
        payload = self._prepare_payload(locals(), exclude=["issue_id", "comment_id"])
        data = await self._client.request(
            method="PATCH",
            uri=f"/issues/{issue_id}/comments/{comment_id}",
            payload=payload,
        )
        return self._decode(Comment, data)

    async def delete_comment(self, issue_id: str, comment_id: str | int) -> bool:
        """Delete issue comment.

        Source:
        https://cloud.yandex.com/en/docs/tracker/concepts/issues/delete-comment
        """
        await self._client.request(
            method="DELETE",
            uri=f"/issues/{issue_id}/comments/{comment_id}",
        )
        return True
